unit uPlannerRole;

interface

uses
  System.SysUtils, System.Generics.Collections,
  Vcl.Dialogs, Vcl.Forms,
  Data.DB,
  AdvOfficePager, AdvOfficeButtons, AdvPanel, DBPlanner, Planner, PlanDraw;

type
  TRolePlanner = class(TObject)
  strict private
  const
    SideBar45 = 45;
    SideBar65 = 65;
  var
    FForm: TForm;
    FDisplayUnit: TList<Integer>;
    FPager: TAdvOfficePager;
    FPlannerDay: TDBPlanner;
    FPositionBar: TList<TAdvOfficeRadioGroup>;
    FPropertiePanel: array[0..7] of TAdvPanel;
    FShapeDrawTool: TObjectList<TShapeDrawTool>;
    FTimeLineUnit: TList<Integer>;
    procedure FillPositionBar;
    procedure FillPropertiePanel;
    procedure GetDisplayUnit;
    procedure GetShapeDrawTool;
    procedure InitialTimeLineUnit;
    procedure SetActiveDay;
    procedure SetActiveDayPeriod;
    procedure SetDisplayStartEnd(const Planner: TDBPlanner);
    procedure SetHalfDayPeriod;
    procedure SetMonth;
    procedure SetMultiDay;
    procedure SetMultiMonth;
    procedure SetPeriod;
    procedure SetTimeLine;
  public
    constructor CreateForm(const AForm: TForm; const ADataSource: TDataSource; AEditPanel: TAdvPanel);
    destructor Destroy; override;
    procedure RepeatSideBar(ARepeat: Boolean);
    procedure SetPlannerProperies;
    property DisplayUnit: TList<Integer>                read FDisplayUnit;
    property ShapeDrawTool: TObjectList<TShapeDrawTool> read FShapeDrawTool;
  end;

implementation

{ TRolePlanner }

uses
   PlannerRole;

constructor TRolePlanner.CreateForm(const AForm: TForm; const ADataSource: TDataSource; AEditPanel: TAdvPanel);
var
  Form: TfrmPlannerRole;
begin
  InitialTimeLineUnit;
  GetDisplayUnit;
  FForm := AForm;
  Form := TfrmPlannerRole(FForm);
  FPager := Form.offpgrPlanner;
  FPlannerDay := Form.dbplDay;
  FillPropertiePanel;
  FillPositionBar;
  GetShapeDrawTool;
end;

destructor TRolePlanner.Destroy;
begin
  FreeAndNil(FPositionBar);
  FreeAndNil(FShapeDrawTool);
  FreeAndNil(FDisplayUnit);
  inherited;
end;

procedure TRolePlanner.FillPositionBar;
var
  Form: TfrmPlannerRole;
begin
  Form := TfrmPlannerRole(FForm);
  FPositionBar := TList<TAdvOfficeRadioGroup>.Create;
  FPositionBar.Add(Form.offrgrSideBarPositionMultiDay);
  FPositionBar.Add(Form.offrgrSideBarPositionPeriod);
  FPositionBar.Add(Form.offrgrSideBarPositionHalfDayPeriod);
  FPositionBar.Add(Form.offrgrSideBarPositionMonth);
  FPositionBar.Add(Form.offrgrSideBarPositionMultiMonth);
  FPositionBar.Add(Form.offrgrSideBarPositionTimeLine);
  FPositionBar.Add(Form.offrgrSideBarPositionActiveDay);
  FPositionBar.Add(Form.offrgrSideBarPositionActiveDayPeriod);
end;

procedure TRolePlanner.FillPropertiePanel;
var
  Form: TfrmPlannerRole;
begin
  Form := TfrmPlannerRole(FForm);
  FPropertiePanel[Ord(TViews(vwMultiDay))] := Form.jstpnlMultiDay;
  FPropertiePanel[Ord(TViews(vwPeriod))] := Form.jstpnlPeriod;
  FPropertiePanel[Ord(TViews(vwHalfDayPeriod))] := Form.jstpnlHalfDayPeriod;
  FPropertiePanel[Ord(TViews(vwMonth))] := Form.jstpnlMonth;
  FPropertiePanel[Ord(TViews(vwMultiMonth))] := Form.jstpnlMultiMonth;
  FPropertiePanel[Ord(TViews(vwTimeLine))] := Form.jstpnlTimeLine;
  FPropertiePanel[Ord(TViews(vwActiveDay))] := Form.jstpnlActiveDay;
  FPropertiePanel[Ord(TViews(vwActiveDayPeriod))] := Form.jstpnlActiveDayPeriod;

end;

procedure TRolePlanner.GetDisplayUnit;
const
  Unit5 = 5;
  Twice = 2;
begin
  FDisplayUnit := TList<Integer>.Create;
  FDisplayUnit.Add(Unit5);
  FDisplayUnit.Add(Unit5 + 1);
  FDisplayUnit.Add(Unit5 * Twice);
  FDisplayUnit.Add(FDisplayUnit.Last + Unit5);
  FDisplayUnit.Add(FDisplayUnit.Last * Twice);
  FDisplayUnit.Add(FDisplayUnit.Last * Twice);
end;

procedure TRolePlanner.GetShapeDrawTool;
begin
  FShapeDrawTool := TObjectList<TShapeDrawTool>.Create(True);
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssCircle;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssTriangle;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssHArrow;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssVArrow;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssTriangles;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssCircles;
  FShapeDrawTool.Add(TShapeDrawTool.Create(nil));
  FShapeDrawTool.Last.Shape := ssSquares;
end;

procedure TRolePlanner.InitialTimeLineUnit;
const
  TimeUnitCount = 2;
var
  I: Integer;
begin
  FTimeLineUnit := TList<Integer>.Create;

  for I := 0 to TimeUnitCount do
    FTimeLineUnit.Add(-1);
end;

procedure TRolePlanner.RepeatSideBar(ARepeat: Boolean);
const
  PositionGap40 = 40;
var
  Form: TfrmPlannerRole;
begin
  Form := TfrmPlannerRole(FForm);

  case TPage(Form.offpgrPlanner.ActivePageIndex) of
    pgDay:
    begin
      FPlannerDay.Sidebar.ShowInPositionGap := ARepeat;
      FPlannerDay.Sidebar.Visible := not ARepeat;
      FPositionBar[Form.advcbxViews.ItemIndex].Enabled := FPlannerDay.Sidebar.Visible;

      if ARepeat then
        FPlannerDay.PositionGap := PositionGap40
      else
        FPlannerDay.PositionGap := 0;
    end;
  end;
end;

procedure TRolePlanner.SetActiveDay;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBActiveDaySource;
begin
  Form := TfrmPlannerRole(FForm);
  SetDisplayStartEnd(FPlannerDay);
  FPlannerDay.Sidebar.Width := SideBar45;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightActiveDay.Value;

  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedActiveDay.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapActiveDay.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionActiveDay.ItemIndex);

  ItemSource := Form.dbacdysrActiveDay;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    Form.SetDislplayStartEnd(Form.offrgrTimeLineActiveDay.Tag, FDisplayUnit[Form.offrgrTimeLineActiveDay.ItemIndex]);
    ItemSource.Day := Form.smdtpkDayActiveDay.Date;
    ItemSource.NumberOfDays := Form.advsedNumberOfDayActiveDay.Value;
    RepeatSideBar(Form.offchbRepeatSideBarActiveDay.Checked);
  finally
    ItemSource.Planner.Update;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetActiveDayPeriod;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBActiveDayPeriodSource;
begin
  Form := TfrmPlannerRole(FForm);
  FPlannerDay.Sidebar.Width := SideBar65;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightActiveDayPeriod.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedActiveDayPeriod.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapActiveDayPeriod.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionActiveDayPeriod.ItemIndex);

  ItemSource := Form.dbacdyprsrActiveDayPeriod;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.StartDate := Form.smdtpkFromActiveDayPeriod.Date;
    ItemSource.EndDate := Form.smdtpkToActiveDayPeriod.Date;
  finally
    ItemSource.UpdatePlanner;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetDisplayStartEnd(const Planner: TDBPlanner);
var
  DisplayEnd: Integer;
begin
  DisplayEnd := (Trunc(MinsPerHour / Planner.Display.DisplayUnit) * HoursPerDay) - 1;
  Planner.Display.DisplayStart := 0;
  Planner.Display.DisplayEnd := DisplayEnd;
end;

procedure TRolePlanner.SetHalfDayPeriod;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBHalfDayPeriodSource;
begin
  Form := TfrmPlannerRole(FForm);
  FPlannerDay.Sidebar.Width := SideBar65;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightHalfDayPeriod.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbInPositionGapHalfDayPeriod.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbShowOccupiedHalfDayPeriod.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionHalfDayPeriod.ItemIndex);

  ItemSource := Form.dbhldyprsrHalfDayPeriod;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.StartDate := Form.smdtpkFromHalfDayPeriod.Date;
    ItemSource.EndDate := Form.smdtpkToHalfDayPeriod.Date;
  finally
    ItemSource.UpdatePlanner;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetMonth;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBMonthSource;
begin
  Form := TfrmPlannerRole(FForm);
  FPlannerDay.Sidebar.Width := SideBar65;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightMonth.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedMonth.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapMonth.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionMonth.ItemIndex);

  ItemSource := Form.dbmnsrMonth;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.Month := Form.advcbxMonthMonth.ItemIndex + 1;
    ItemSource.Year := Form.advsedYearMonth.Value;
  finally
    ItemSource.Planner.Update;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetMultiDay;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBDaySource;
begin
  Form := TfrmPlannerRole(FForm);
  SetDisplayStartEnd(FPlannerDay);
  FPlannerDay.Sidebar.Width := SideBar45;

  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedMultiDay.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapMultiDay.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionMultiDay.ItemIndex);

  ItemSource := Form.dbdysrMultiDay;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    Form.SetDislplayStartEnd(Form.offrgrTimeLineMultiDay.Tag, FDisplayUnit[Form.offrgrTimeLineMultiDay.ItemIndex]);
    FPlannerDay.Display.DisplayScale := Form.advsedRowHeightMultiDay.Value;
    ItemSource.Day := Form.smdtpkDayMultiDay.Date;
    ItemSource.NumberOfDays := Form.advsedNumberOfDayMultiDay.Value;
    ItemSource.DayIncrement := Form.advsedDayIncrementMultiDay.Value;
    RepeatSideBar(Form.offchbRepeatSideBarMultiDay.Checked);
  finally
    ItemSource.Planner.Update;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetMultiMonth;
const
  SideBar20 = 20;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBMultiMonthSource;
begin
  Form := TfrmPlannerRole(FForm);
  FPlannerDay.Sidebar.Width := SideBar20;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightMultiMonth.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedMultiMonth.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapMultiMonth.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionMultiMonth.ItemIndex);

  ItemSource := Form.dbmlmnsrMultiMonth;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.StartMonth := Form.advcbxStartMonthMultiMonth.ItemIndex + 1;
    ItemSource.Year := Form.advsedYearMultiMonth.Value;
    ItemSource.NumberOfMonths := Form.advsedMonthOfMonthMultiMonth.Value;
    ItemSource.MonthIncrement := Form.advsedMonthIncrementMultiMonth.Value;
    RepeatSideBar(Form.offchbRepeatSideBarMultiMonth.Checked);
  finally
    ItemSource.UpdatePlanner;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetPeriod;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBPeriodSource;
begin
  Form := TfrmPlannerRole(FForm);
  FPlannerDay.Sidebar.Width := SideBar65;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightPeriod.Value;
  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedPeriod.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapPeriod.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionPeriod.ItemIndex);

  ItemSource := Form.dbprsrPeriod;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    ItemSource.StartDate := Form.smdtpkFromPeriod.Date;
    ItemSource.EndDate := Form.smdtpkToPeriod.Date;
  finally
    ItemSource.UpdatePlanner;
    ItemSource.EndUpdate;
  end;
end;

procedure TRolePlanner.SetPlannerProperies;
var
  I: Integer;
  Form: TfrmPlannerRole;
begin
  Form := TfrmPlannerRole(FForm);

  for I := Low(FPropertiePanel) to High(FPropertiePanel) do
    FPropertiePanel[I].Visible := False;

  FPropertiePanel[Form.advcbxViews.ItemIndex].Visible := True;

  case TPage(FPager.ActivePageIndex) of
    pgDay:
    begin
      case TViews(Form.advcbxViews.ItemIndex) of
      vwMultiDay: SetMultiDay;
      vwPeriod: SetPeriod;
      vwHalfDayPeriod: SetHalfDayPeriod;
      vwMonth: SetMonth;
      vwMultiMonth: SetMultiMonth;
      vwTimeLine: SetTimeLine;
      vwActiveDay: SetActiveDay;
      vwActiveDayPeriod: SetActiveDayPeriod;
      end;
    end;
  end;
end;

procedure TRolePlanner.SetTimeLine;
var
  Form: TfrmPlannerRole;
  ItemSource: TDBTimeLineSource;
begin
  Form := TfrmPlannerRole(FForm);
  FPlannerDay.Sidebar.Width := SideBar65;
  FPlannerDay.Display.DisplayScale := Form.advsedRowHeightTimeLine.Value;

  FPlannerDay.Sidebar.ShowOccupied := Form.offchbShowOccupiedTimeLine.Checked;
  FPlannerDay.ShowOccupiedInPositionGap := Form.offchbInPositionGapTimeLine.Checked;
  FPlannerDay.SideBar.Position := TSideBarPosition(Form.offrgrSideBarPositionTimeLine.ItemIndex);

  ItemSource := Form.dbtmlnsrTimeLine;
  FPlannerDay.ItemSource := ItemSource;
  ItemSource.BeginUpdate;

  try
    Form.SetDislplayStartEnd(Form.offrgrTimeLineTimeLine.Tag, FDisplayUnit[Form.offrgrTimeLineTimeLine.ItemIndex]);
    ItemSource.StartDate := Form.smdtpkFromTimeLine.Date;
    ItemSource.EndDate := Form.smdtpkToTimeLine.Date;
  finally
    ItemSource.UpdatePlanner;
    ItemSource.EndUpdate;
  end;
end;

end.
